@@ -108,7 +108,7 @@ class HALF_EXPORT half
     //-------------
 
     half ();			// no initialization
-    explicit half (float f);
+    half (float f);
 
 
     //--------------------
