@@ -32,8 +32,6 @@
 #include <jellyfish/storage.hpp>
 #include <jellyfish/offsets_key_value.hpp>
 #include <jellyfish/parse_dna.hpp>
-#include <jellyfish/misc.hpp>
-#include <jellyfish/simple_circular_buffer.hpp>
 
 namespace jellyfish {
   namespace invertible_hash {
@@ -297,8 +295,9 @@ namespace jellyfish {
       friend class iterator;
       iterator iterator_all() const { return iterator(this, 0, get_size()); }
       iterator iterator_slice(size_t slice_number, size_t number_of_slice) const {
-        std::pair<size_t, size_t> res = slice(slice_number, number_of_slice, get_size());
-        return iterator(this, res.first, res.second);
+        size_t slice_size = get_size() / number_of_slice;
+        size_t start = slice_number * slice_size;
+        return iterator(this, start, start + slice_size);
       }
 
       /* Why on earth doesn't inheritance with : public iterator work
@@ -451,8 +450,7 @@ namespace jellyfish {
        * forward for entries with large bit set pointing back to the
        * key at id, and all those values are summed up.
        */
-      bool get_key_val_full(size_t id, word &key, word &val,
-                            bool carry_bit = false) const {
+      bool get_key_val_full(size_t id, word &key, word &val) const {
         const offset_t	*o, *lo;
         word		*w, *kvw, nkey, nval;
         uint_t		 reprobe = 0, overflows = 0;
@@ -479,13 +477,6 @@ namespace jellyfish {
         if(o->val.mask2)
           val |= ((*(kvw+1)) & o->val.mask2) << o->val.shift;
 
-        if(carry_bit) {
-          bool do_reprobe = val & 0x1;
-          val >>= 1;
-          if(!do_reprobe)
-            return true;
-        }
-
         // Resolve value
         reprobe = 0;
         cid = id = (id + reprobes[0]) & size_mask;
@@ -508,19 +499,10 @@ namespace jellyfish {
               nval = ((*kvw) & lo->val.mask1) >> lo->val.boff;
               if(lo->val.mask2)
                 nval |= ((*(kvw+1)) & lo->val.mask2) << lo->val.shift;
-              bool do_reprobe = true;
-              if(carry_bit) {
-                do_reprobe = nval & 0x1;
-                nval >>= 1;
-              }
-
               nval <<= offsets.get_val_len();
               nval <<= offsets.get_lval_len() * overflows;
               val += nval;
 
-              if(!do_reprobe)
-                return true;
-
               overflows++;
               reprobe = 0;
               cid = id = (cid + reprobes[0]) & size_mask;
@@ -542,125 +524,31 @@ namespace jellyfish {
         return true;
       }
 
-      inline bool get_val(const word key, word &val, bool full = false,
-                          bool carry_bit = false) const {
+      inline bool get_val(const word key, word &val, bool full = false) const {
         uint64_t hash = hash_matrix.times(key);
         size_t key_id;
-        return _get_val(hash & size_mask, key_id, (hash >> lsize) & key_mask, val, 
-                        full, carry_bit);
+        return _get_val(hash & size_mask, key_id, (hash >> lsize) & key_mask, val, full);
       }
 
-      inline bool get_val(const word key, size_t &key_id, word &val, 
-                          bool full = false, bool carry_bit = false) const {
+      inline bool get_val(const word key, size_t &key_id, word &val, bool full = false) const {
         uint64_t hash = hash_matrix.times(key);
-        return _get_val(hash & size_mask, key_id, (hash >> lsize) & key_mask, val,
-                        full, carry_bit);
-      }
-
-      // /* InputIterator points to keys (words). OutputIterator points
-      //    to struct containing at least the fields { bool found; size_t
-      //    key_id; word val; }.
-      //  */
-      // template<typename InputIterator, typename OutputIterator>
-      // void get_multi_val(InputIterator key, const InputIterator& key_end,
-      //                    OutputIterator val, bool full, bool carry_bit) const {
-      //   uint64_t        phash, chash;
-      //   const offset_t *po, *plo, *co, *clo;
-      //   const word     *pw, *cw;
-
-      //   if(key == key_end)
-      //     return;
-
-      //   // Call __get_val with a delay. Let prefetch work while we
-      //   // compute the hash/get the previous key.
-      //   phash = hash_matrix.times(*key);
-      //   pw    = offsets.get_word_offset(phash & size_mask, &po, &plo, data);
-      //   //__builtin_prefetch(pw + po->key.woff, 0, 3);
-
-      //   for(++key; key != key_end; ++key, ++val) {
-      //     chash = hash_matrix.times(*key);
-      //     cw    = offsets.get_word_offset(chash & size_mask, &co, &clo, data);
-      //     //__builtin_prefetch(cw + co->key.woff, 0, 3);
-
-      //     val->found = __get_val(phash & size_mask, val->key_id, 
-      //                            (phash >> lsize) & key_mask, val->val,
-      //                            full, carry_bit, pw, po, plo);
-
-
-      //     pw    = cw;
-      //     po    = co;
-      //     plo   = clo;
-      //     phash = chash;
-      //   }
-      //   // Last one
-      //   val->found =  __get_val(phash & size_mask, val->key_id, 
-      //                           (phash >> lsize) & key_mask, val->val,
-      //                           full, carry_bit, pw, po, plo);
-      // }
-
-      struct prefetch_info {
-        const word*     w;
-        const offset_t *o, *lo;
-      };
-      typedef simple_circular_buffer::pre_alloc<prefetch_info, 8> prefetch_buffer;
-
-      void warm_up_cache(prefetch_buffer& buffer, size_t id, bool load_lo) const {
-        buffer.clear();
-        for(int i = 0; i < buffer.capacity(); ++i) {
-          buffer.push_back();
-          prefetch_info& info = buffer.back();
-          size_t         cid  = (id + (i > 0 ? reprobes[i] : 0)) & size_mask;
-          info.w              = offsets.get_word_offset(cid, &info.o, &info.lo, data);
-          __builtin_prefetch(info.w + info.o->key.woff, 0, 1);
-          __builtin_prefetch(info.o, 0, 3);
-          if(load_lo)
-            __builtin_prefetch(info.lo, 0, 3);
-        }
-      }
-
-      void prefetch_next(prefetch_buffer& buffer, size_t id, uint_t reprobe, bool load_lo) const {
-        buffer.pop_front();
-        if(reprobe + buffer.capacity() <= reprobe_limit.val()) {
-          buffer.push_back();
-          prefetch_info& info = buffer.back();
-          size_t         fid  = (id + reprobes[reprobe + buffer.capacity() - 1]) & size_mask;
-          info.w = offsets.get_word_offset(fid, &info.o, &info.lo, data);
-          __builtin_prefetch(info.w + info.o->key.woff, 0, 1);
-          __builtin_prefetch(info.o, 0, 3);
-          if(load_lo)
-            __builtin_prefetch(info.lo, 0, 3);
-        }
-
+        return _get_val(hash & size_mask, key_id, (hash >> lsize) & key_mask, val, full);
       }
 
       bool _get_val(const size_t id, size_t &key_id, const word key, word &val, 
-                           bool full = false, bool carry_bit = false) const {
-        // Buffer for pre-cached information
-        prefetch_info info_ary[prefetch_buffer::capacity()];
-        prefetch_buffer buffer(info_ary);
-        warm_up_cache(buffer, id, false);
-
-        return __get_val(id, key_id, key, val, full, carry_bit, buffer);
-      }
-
-      bool __get_val(const size_t id, size_t &key_id, const word key, word &val, 
-                     const bool full, bool carry_bit,
-                     prefetch_buffer& buffer) const {
-        const word     *kvw;
-        word            nkey, nval;
+                   const bool full = false) const {
+        word           *w, *kvw, nkey, nval;
+        const offset_t *o, *lo;
         size_t          cid     = id;
         uint_t          reprobe = 0;
         word            akey    = key | ((word)1 << key_off);
 
+        //        std::cout << "_get_val key" << key << std::endl;
         // Find key
-        const offset_t *o, *lo;
-        const word* w;
         while(true) {
-          prefetch_info& info = buffer.front();
-          w                   = info.w;
-          o                   = info.o;
-          kvw                 = w + o->key.woff;
-          nkey                = *kvw;
+          w   = offsets.get_word_offset(cid, &o, &lo, data);
+          kvw = w + o->key.woff;
+          nkey = *kvw;
       
           if(!(nkey & o->key.lb_mask)) {
             if(o->key.mask2) {
@@ -674,11 +562,8 @@ namespace jellyfish {
           }
           if(++reprobe > reprobe_limit.val())
             return false;
-          // Do reprobe
-          cid  = (id + reprobes[reprobe]) & size_mask;
+          cid = (id + reprobes[reprobe]) & size_mask;
           akey = key | ((reprobe + 1) << key_off);
-
-          prefetch_next(buffer, id, reprobe, false);
         }
 
         // Get value
@@ -687,30 +572,17 @@ namespace jellyfish {
         if(o->val.mask2) {
           val |= ((*(kvw+1)) & o->val.mask2) << o->val.shift;
         }
-        bool do_reprobe = true;
-        if(carry_bit) {
-          do_reprobe   = val & 0x1;
-          val        >>= 1;
-        }
         key_id = cid;
 
-        // Eventually get large values...  TODO: this seems buggy. It
-        // only looks for large values once, not as many times as
-        // needed.
-        if(full && do_reprobe) {
+        // Eventually get large values...
+        if(full) {
           const size_t bid = (cid + reprobes[0]) & size_mask;
           cid = bid;
-
-          warm_up_cache(buffer, bid, true);
-
           reprobe = 0;
           do {
-            prefetch_info& info = buffer.front();
-            const word*    w    = info.w;
-            o                   = info.o;
-            lo                  = info.lo;
-            kvw                 = w + o->key.woff;
-            nkey                = *kvw;
+            w   = offsets.get_word_offset(cid, &o, &lo, data);
+            kvw = w + o->key.woff;
+            nkey = *kvw;
             if(nkey & o->key.lb_mask) {
               if(lo->key.mask2) {
                 nkey = (nkey & lo->key.mask1 & ~lo->key.sb_mask1) >> lo->key.boff;
@@ -723,20 +595,13 @@ namespace jellyfish {
                 nval = ((*kvw) & lo->val.mask1) >> lo->val.boff;
                 if(lo->val.mask2)
                   nval |= ((*(kvw+1)) & lo->val.mask2) << lo->val.shift;
-                if(carry_bit) {
-                  nval >>= 1;
-                  val |= nval << (offsets.get_val_len() - 1);
-                } else
-                  val |= nval << offsets.get_val_len();
-                break; // Should break only if carry_bit of nval is
-                       // not set. Otherwise, we should reset the
-                       // reprobe to 0 and try again.
+
+                val |= nval << offsets.get_val_len();
+                break;
               }
             }
 
             cid = (bid + reprobes[++reprobe]) & size_mask;
-            
-            prefetch_next(buffer, bid, reprobe, true);
           } while(reprobe <= reprobe_limit.val());
         }
 
@@ -752,8 +617,6 @@ namespace jellyfish {
                        val, false, oval);
       }
 
-      
-
       /**
        * Use hash as a set.
        */
@@ -777,24 +640,6 @@ namespace jellyfish {
                          &ao, &w);
       }
 
-      /**
-       * Use hash as a map. This sets a value with the key. It is only
-       * partially thread safe. I.e., multiple different key can be
-       * added concurrently. On the other hand, the same key can not
-       * be added at the same time by different thread: the value set
-       * may not be correct.
-       */
-      inline bool map(word _key, word val) {
-        bool   is_new;
-        return map(_key, val, &is_new);
-      }
-
-      bool map(word _key, word val, bool* is_new) {
-        uint64_t hash = hash_matrix.times(_key);
-        return map_rec(hash & size_mask, (hash >> lsize) & key_mask, val, false, 
-                       is_new);
-      }
-
       void write_ary_header(std::ostream *out) const {
         hash_matrix.dump(out);
         hash_inverse_matrix.dump(out);
@@ -908,43 +753,6 @@ namespace jellyfish {
         return true;
       }
 
-      // Store val in the hash at position id. Reprobe and recurse if
-      // val does not fit in counter field. A bit is added at the
-      // beginning of the counting field indicating whether there is
-      // another entry for the same key further in the hash.
-      bool map_rec(size_t id, word key, word val, bool large, bool* is_new) {
-        const offset_t* ao;
-        word*           w;
-        bool            is_new_entry = false;
-        if(!claim_key(key, &is_new_entry, &id, large, &ao, &w))
-          return false;
-        if(is_new)
-          *is_new = is_new_entry;
-
-        // Determine if there will be a carry
-        val <<= 1;
-        val |= (val > offsets.get_max_val(large));
-
-        // Set value
-        word *vw         = w + ao->val.woff;
-        word  oval       = 0;
-        word  cary       = set_val(vw, val, ao->val.boff, ao->val.mask1, oval);
-        cary           >>= ao->val.shift;
-        bool  cary_bit   = oval & 0x1;
-        if(ao->val.mask2) { // value split on two words. Write even if
-                            // cary is 0 as there maybe some value in
-                            // there already
-          cary   = set_val(vw + 1, cary, 0, ao->val.mask2, oval);
-          cary >>= ao->val.cshift;
-        }
-        // Done if there is no carry and previous value did not have
-        // the carry_bit set
-        if(!cary && !cary_bit)
-          return true;
-        id = (id + reprobes[0]) & size_mask;
-        return map_rec(id, key, cary, true, 0);
-      }
-
       inline bool set_key(word *w, word nkey, word free_mask, 
                           word equal_mask) {
         word ow = *w, nw, okey;
@@ -991,21 +799,6 @@ namespace jellyfish {
 
         return nval & (~(mask >> shift));
       }
-
-      inline word set_val(word *w, word val, uint_t shift, word mask,
-                          word& oval) {
-        word now = *w, ow, nw;
-        word sval = (val << shift) & mask;
-
-        do {
-          ow = now;
-          nw = (ow & ~mask) | sval;
-          now = atomic.cas(w, ow, nw);
-        } while(now != ow);
-
-        oval = (ow & mask) >> shift;
-        return val & (~(mask >> shift));
-      }
     };
 
     /*****/
